@@ -280,7 +280,7 @@ class DQFDAgent(Agent):
             # Sample from demo memory.
             records, sample_indices, importance_weights = agent.demo_memory.get_records(demo_batch_size)
             preprocessed_s, actions, rewards, terminals, preprocessed_s_prime = agent.splitter.call(records)
-            step_op, loss, loss_per_item, q_values_s = root.update_from_external_batch(
+            step_op, loss, loss_per_item = root.update_from_external_batch(
                 preprocessed_s, actions, rewards, terminals, preprocessed_s_prime, importance_weights, apply_demo_loss,
                 expert_margins, time_percentage
             )
@@ -326,7 +326,7 @@ class DQFDAgent(Agent):
 
             loss, loss_per_item = loss_function.loss(
                 q_values_s, actions, rewards, terminals, qt_values_sp, expert_margins, q_values_sp,
-                importance_weights, apply_demo_loss, time_percentage
+                importance_weights, apply_demo_loss
             )
 
             # Args are passed in again because some device strategies may want to split them to different devices.
@@ -359,11 +359,12 @@ class DQFDAgent(Agent):
 
             loss, loss_per_item = loss_function.loss(
                 q_values_s, actions, rewards, terminals, qt_values_sp, expert_margins,
-                q_values_sp, importance_weights, apply_demo_loss, time_percentage
+                q_values_sp, importance_weights, apply_demo_loss
             )
             return loss, loss_per_item
 
-    def get_action(self, states, internals=None, use_exploration=True, apply_preprocessing=True, extra_returns=None):
+    def get_action(self, states, internals=None, use_exploration=True, apply_preprocessing=True, extra_returns=None,
+                   time_percentage=None):
         """
         Args:
             extra_returns (Optional[Set[str],str]): Optional string or set of strings for additional return
@@ -378,6 +379,10 @@ class DQFDAgent(Agent):
                 - action
                 - the preprocessed states
         """
+        # TODO: Move update_spec to Worker. Agent should not hold these execution details.
+        if time_percentage is None:
+            time_percentage = self.timesteps / self.update_spec.get("max_timesteps", 1e6)
+
         extra_returns = {extra_returns} if isinstance(extra_returns, str) else (extra_returns or set())
         # States come in without preprocessing -> use state space.
         if apply_preprocessing:
@@ -396,7 +401,7 @@ class DQFDAgent(Agent):
         return_ops = [0, 1] if "preprocessed_states" in extra_returns else [0]  # 1=preprocessed_states, 0=action
         ret = self.graph_executor.execute((
             call_method,
-            [batched_states, self.timesteps, use_exploration],
+            [batched_states, time_percentage, use_exploration],
             return_ops
         ))
         if remove_batch_rank:
