@@ -18,6 +18,7 @@ from __future__ import division
 from __future__ import print_function
 
 import numpy as np
+
 from rlgraph import get_backend
 from rlgraph.components.component import Component
 from rlgraph.utils.decorators import rlgraph_api
@@ -43,7 +44,7 @@ class VTraceFunction(Component):
             rho_bar (float): The maximum values of the IS-weights for the temporal differences of V.
                 Use None for not applying any clipping.
             rho_bar_pg (float): The maximum values of the IS-weights for the policy-gradient loss:
-                \rho_s \delta log \pi(a|x) (r + \gamma v_{s+1} - V(x_s))
+                rho_s delta log pi(a|x) (r + gamma v_{s+1} - V(x_s))
                 Use None for not applying any clipping.
             c_bar (float): The maximum values of the IS-weights for the time trace.
                 Use None for not applying any clipping.
