@@ -18,9 +18,10 @@ from __future__ import division
 from __future__ import print_function
 
 import numpy as np
+
 from rlgraph import get_backend
 from rlgraph.components.loss_functions import LossFunction
-from rlgraph.spaces import IntBox, ContainerSpace
+from rlgraph.spaces import IntBox
 from rlgraph.spaces.space_utils import sanity_check_space
 from rlgraph.utils import pytorch_one_hot
 from rlgraph.utils.decorators import rlgraph_api, graph_fn
