@@ -19,26 +19,26 @@ from __future__ import print_function
 
 from rlgraph.environments.environment import Environment
 from rlgraph.environments.deterministic_env import DeterministicEnv
-from rlgraph.environments.gaussian_density_as_reward_env import GaussianDensityAsRewardEnvironment
+from rlgraph.environments.gaussian_density_as_reward_env import GaussianDensityAsRewardEnv
 from rlgraph.environments.grid_world import GridWorld
 from rlgraph.environments.openai_gym import OpenAIGymEnv
 from rlgraph.environments.random_env import RandomEnv
 from rlgraph.environments.vector_env import VectorEnv
 from rlgraph.environments.sequential_vector_env import SequentialVectorEnv
 
-
 Environment.__lookup_classes__ = dict(
     deterministic=DeterministicEnv,
     deterministicenv=DeterministicEnv,
-    gaussiandensity=GaussianDensityAsRewardEnvironment,
-    gaussiandensityasreward=GaussianDensityAsRewardEnvironment,
-    gaussiandensityasrewardenv=GaussianDensityAsRewardEnvironment,
+    gaussiandensity=GaussianDensityAsRewardEnv,
+    gaussiandensityasreward=GaussianDensityAsRewardEnv,
+    gaussiandensityasrewardenv=GaussianDensityAsRewardEnv,
     gridworld=GridWorld,
+    gridworldenv=GridWorld,
     openai=OpenAIGymEnv,
-    openaigymenv=OpenAIGymEnv,
     openaigym=OpenAIGymEnv,
-    randomenv=RandomEnv,
+    openaigymenv=OpenAIGymEnv,
     random=RandomEnv,
+    randomenv=RandomEnv,
     sequentialvector=SequentialVectorEnv,
     sequentialvectorenv=SequentialVectorEnv
 )
@@ -58,5 +58,20 @@ except Exception:
     pass
 
 
+try:
+    import mlagents
+
+    # If import works: Can import our Adapter.
+    from rlgraph.environments.mlagents_env import MLAgentsEnv
+
+    Environment.__lookup_classes__.update(dict(
+        mlagents=MLAgentsEnv,
+        mlagentsenv=MLAgentsEnv,
+    ))
+    # TODO travis error on this, investigate.
+except Exception:
+    pass
+
+
 __all__ = ["Environment"] + \
           list(set(map(lambda x: x.__name__, Environment.__lookup_classes__.values())))
