@@ -18,20 +18,19 @@ from __future__ import division
 from __future__ import print_function
 
 import logging
-import unittest
 import time
+import unittest
 
-from rlgraph import get_backend
 from rlgraph.agents import DQNAgent, ApexAgent
 from rlgraph.components import Policy, MemPrioritizedReplay
 from rlgraph.environments import OpenAIGymEnv
 from rlgraph.spaces import FloatBox, IntBox, Dict, BoolBox
 from rlgraph.tests import ComponentTest
-from rlgraph.tests.test_util import config_from_path
-from rlgraph.utils import root_logger, softmax
 from rlgraph.tests.dummy_components import *
 from rlgraph.tests.dummy_components_with_sub_components import *
-from rlgraph.utils.execution_util import print_call_chain
+from rlgraph.tests.test_util import config_from_path
+from rlgraph.utils import root_logger, softmax
+from rlgraph.utils.define_by_run_ops import print_call_chain
 
 
 class TestPytorchBackend(unittest.TestCase):
@@ -154,7 +153,7 @@ class TestPytorchBackend(unittest.TestCase):
         # Batch of size=1 (can increase this to any larger number).
         input_ = np.array([0.5, 2.0])
         expected = np.array([2.5, 2.5])
-        test.test(("apply", input_), expected_outputs=expected)
+        test.test(("call", input_), expected_outputs=expected)
 
     def test_nn_assembly_from_file(self):
         # Space must contain batch dimension (otherwise, NNlayer will complain).
@@ -171,7 +170,7 @@ class TestPytorchBackend(unittest.TestCase):
 
         # Cant fetch variables here.
 
-        out = test.test(("apply", input_), decimals=5)
+        out = test.test(("call", input_), decimals=5)
         print(out)
 
     def test_policy_for_discrete_action_space(self):
@@ -193,23 +192,26 @@ class TestPytorchBackend(unittest.TestCase):
         states = np.array([[-0.08, 0.4, -0.05, -0.55], [13.0, -14.0, 10.0, -16.0]])
         # Raw NN-output.
         expected_nn_output = np.matmul(states, policy_params["policy/test-network/hidden-layer/dense/kernel"])
-        test.test(("get_nn_output", states), expected_outputs=dict(output=expected_nn_output), decimals=6)
+        test.test(("get_nn_output", states), expected_outputs=expected_nn_output, decimals=6)
 
         # Raw action layer output; Expected shape=(2,5): 2=batch, 5=action categories
         expected_action_layer_output = np.matmul(
             expected_nn_output, policy_params["policy/action-adapter/action-layer/dense/kernel"]
         )
         expected_action_layer_output = np.reshape(expected_action_layer_output, newshape=(2, 5))
-        test.test(("get_action_layer_output", states), expected_outputs=dict(output=expected_action_layer_output),
-                  decimals=5)
+        test.test(
+            ("get_adapter_outputs", states, ["adapter_outputs"]),
+            expected_outputs=dict(adapter_outputs=expected_action_layer_output),
+            decimals=5
+        )
 
         expected_actions = np.argmax(expected_action_layer_output, axis=-1)
-        test.test(("get_action", states), expected_outputs=dict(action=expected_actions, last_internal_states=None))
+        test.test(("get_action", states, ["action"]), expected_outputs=dict(action=expected_actions))
 
         # Logits, parameters (probs) and skip log-probs (numerically unstable for small probs).
         expected_probabilities_output = softmax(expected_action_layer_output, axis=-1)
-        test.test(("get_logits_parameters_log_probs", states, [0, 1, 2]), expected_outputs=dict(
-            logits=expected_action_layer_output,
+        test.test(("get_adapter_outputs_and_parameters", states, [0, 1, 2]), expected_outputs=dict(
+            adapter_outputs=expected_action_layer_output,
             parameters=expected_probabilities_output,
             log_probs=np.log(expected_probabilities_output)
         ), decimals=5)
