@@ -27,7 +27,7 @@ from rlgraph.utils.util import convert_dtype, get_shape, get_rank, force_tuple,
     tf_logger, print_logging_handler, root_logger, logging_formatter, default_dict
 from rlgraph.utils.numpy import softmax, relu, one_hot
 from rlgraph.utils.pytorch_util import pytorch_one_hot, PyTorchVariable
-from rlgraph.utils.execution_util import print_call_chain
+from rlgraph.utils.define_by_run_ops import print_call_chain
 # from rlgraph.utils.specifiable_server import SpecifiableServer, SpecifiableServerHook
 #from rlgraph.utils.decorators import api
 
