@@ -13,9 +13,7 @@
 # limitations under the License.
 # ==============================================================================
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
+from __future__ import absolute_import, division, print_function
 
 import inspect
 
@@ -50,7 +48,7 @@ class DataOpRecord(object):
             space (Optional[Space]): The Space of `op` if already known at construction time. Will be poulated
                 later (during build phase) if not.
 
-            next\_ (Optional(Set[DataOpRecord],DataOpRecord)): The next op-record or set of op-records.
+            next_ (Optional(Set[DataOpRecord],DataOpRecord)): The next op-record or set of op-records.
             previous (Optional(DataOpRecord)): The previous op-record.
         """
         self.id = self.get_id()
@@ -115,6 +113,10 @@ class DataOpRecord(object):
 
 
 class DataOpRecordColumn(object):
+    """
+    A DataOpRecordColumn is a list of DataOpRecords that either go into (a call) or come from (return) a
+    Component's GraphFn or API method.
+    """
     _ID = -1
 
     def __init__(self, component, num_op_records=None, args=None, kwargs=None):
